@@ -65,3 +65,29 @@ module M = struct
   type a = A of b [@@deriving compare]
   and b = B of a
 end
+
+(* #275 *)
+let g x =
+  (x * x
+   [@ocaml.ppwarning "TODO: blabla"])
+
+let h = "I am well indented"
+
+let i x =
+  x * x
+  [@ocaml.ppwarning "TODO: blabla"]
+
+let j = "I am NOT well indented"
+
+(* #277 *)
+module V = struct
+  type t =
+  | A of A.t [@blah "a"]
+  | B of B.t [@blah "b"]
+  | C of C.t [@blah "c"]
+end
+
+let foo =
+  let f x =
+    foo bar [@@bla] in
+  zz
